#include "StdAfx.h"
#include "Wininet.h"

CWininet::CWininet(void)
{
	m_hConnectedEvent		= CreateEvent(NULL, FALSE, FALSE, NULL);
    m_hRequestOpenedEvent	= CreateEvent(NULL, FALSE, FALSE, NULL);
    m_hRequestCompleteEvent	= CreateEvent(NULL, FALSE, FALSE, NULL);

	m_hInstance	= NULL;
	m_hConnect	= NULL;
	m_hRequest	= NULL;
}

CWininet::~CWininet(void)
{
	if (m_hConnectedEvent)
		CloseHandle(m_hConnectedEvent);
	if (m_hRequestOpenedEvent)
		CloseHandle(m_hRequestOpenedEvent);
	if (m_hRequestCompleteEvent)
		CloseHandle(m_hRequestCompleteEvent);

	Close();
}

void WINAPI CWininet::Callback(HINTERNET hInternet,
				  DWORD dwContext,
				  DWORD dwInternetStatus,
				  LPVOID lpStatusInfo,
				  DWORD dwStatusInfoLen)
{
	SContext* pContext = (SContext*) dwContext;

	switch(pContext->dwContext)
	{
	case CONTEXT_CONNECT:
		if (dwInternetStatus == INTERNET_STATUS_HANDLE_CREATED)
		{
			INTERNET_ASYNC_RESULT *pRes = (INTERNET_ASYNC_RESULT *)lpStatusInfo;
			pContext->pObj->m_hConnect = (HINTERNET)pRes->dwResult;
			SetEvent(pContext->pObj->m_hConnectedEvent);
		}
		break;

	case CONTEXT_REQUESTHANDLE: // Request handle
		{
			switch(dwInternetStatus)
			{
			case INTERNET_STATUS_HANDLE_CREATED:
				{
					INTERNET_ASYNC_RESULT *pRes = (INTERNET_ASYNC_RESULT *)lpStatusInfo;
					pContext->pObj->m_hRequest = (HINTERNET)pRes->dwResult;
					SetEvent(pContext->pObj->m_hRequestOpenedEvent);
				}
				break;

			case INTERNET_STATUS_REQUEST_SENT:
				{
					DWORD *lpBytesSent = (DWORD*)lpStatusInfo;
				}
				break;

			case INTERNET_STATUS_REQUEST_COMPLETE:
				{
					INTERNET_ASYNC_RESULT *pAsyncRes = (INTERNET_ASYNC_RESULT *)lpStatusInfo;
					SetEvent(pContext->pObj->m_hRequestCompleteEvent);
				}
				break;

			case INTERNET_STATUS_REDIRECT:
				//string strRealAddr = (LPSTR) lpStatusInfo;
				break;

			case INTERNET_STATUS_RECEIVING_RESPONSE:
				break;

			case INTERNET_STATUS_RESPONSE_RECEIVED:
				{
					DWORD *dwBytesReceived = (DWORD*)lpStatusInfo;
					//if (*dwBytesReceived == 0)
					//    bAllDone = TRUE;

				}
			}
		}
	}
}

void CWininet::Close()
{
	if (m_hInstance)
	{
		InternetCloseHandle(m_hInstance);
		m_hInstance = NULL;
	}

	if (m_hConnect)
	{
		InternetCloseHandle(m_hConnect);
		m_hConnect = NULL;
	}

	if (m_hRequest)
	{
		InternetCloseHandle(m_hRequest);
		m_hRequest = NULL;
	}
}


BOOL CWininet::Connect(LPCTSTR lpszAddr, USHORT uPort, LPCTSTR lpszAgent, DWORD dwTimeOut)
{
	Close();

	ResetEvent(m_hConnectedEvent);
	ResetEvent(m_hRequestOpenedEvent);
	ResetEvent(m_hRequestCompleteEvent);


	if (!(m_hInstance = InternetOpen(lpszAgent, 
					INTERNET_OPEN_TYPE_PRECONFIG,
					NULL,
					NULL,
					INTERNET_FLAG_ASYNC)))
	{
        return FALSE;
	}


	if (InternetSetStatusCallback(m_hInstance,
				(INTERNET_STATUS_CALLBACK)&Callback)
				== INTERNET_INVALID_STATUS_CALLBACK)
	{
		return FALSE;
	}


	m_context.dwContext	= CONTEXT_CONNECT;
	m_context.pObj		= this;

	m_hConnect = InternetConnect(m_hInstance, 
					lpszAddr,
					uPort,
					NULL,
					NULL,
					INTERNET_SERVICE_HTTP,
					INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE,
					(DWORD)&m_context);


	if (m_hConnect == NULL)
	{
		if (GetLastError() != ERROR_IO_PENDING)
			return FALSE;

		if(WaitForSingleObject(m_hConnectedEvent, dwTimeOut) == WAIT_TIMEOUT)
			return FALSE;
	}

	if(m_hConnect == NULL)
		return FALSE;


	return TRUE;
}

BOOL CWininet::SendRequest(LPCTSTR szURI, LPCTSTR lpszReferrer, DWORD dwTimeOut)
{

	LPCTSTR szAcceptType = _T("*/*");


	m_context.dwContext	= CONTEXT_REQUESTHANDLE;
	m_context.pObj		= this;

	m_hRequest = HttpOpenRequest(m_hConnect, 
					_T("GET"),
					szURI,
					NULL,
					lpszReferrer,
					NULL,
					INTERNET_FLAG_RELOAD | INTERNET_FLAG_KEEP_CONNECTION 
					| INTERNET_FLAG_NO_CACHE_WRITE | INTERNET_FLAG_FORMS_SUBMIT 
					| INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS 
					| INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP,
					(DWORD) &m_context); 


	if (m_hRequest == NULL)
	{
		if (GetLastError() != ERROR_IO_PENDING)
			return FALSE;

		if (WaitForSingleObject(m_hRequestOpenedEvent, dwTimeOut) == WAIT_TIMEOUT)
			return FALSE;
	}


	if (m_hRequest == NULL)
		return FALSE;


	if (!HttpSendRequest(m_hRequest, 
					 NULL, 
					 0, 
					 NULL,
					 0))
	{
		if (GetLastError() != ERROR_IO_PENDING)
			return FALSE;

	}


	if (WaitForSingleObject(m_hRequestCompleteEvent, dwTimeOut) == WAIT_TIMEOUT)
	{
		Close();
		return FALSE;
	}


	return TRUE;
}

DWORD CWininet::Read(PBYTE pBuffer, DWORD dwSize, DWORD dwTimeOut)
{
	INTERNET_BUFFERS InetBuff;

	FillMemory(&InetBuff, sizeof(InetBuff), 0);

	InetBuff.dwStructSize	= sizeof(InetBuff);
	InetBuff.lpvBuffer		= pBuffer;
	InetBuff.dwBufferLength	= dwSize - 1;


	m_context.dwContext	= CONTEXT_REQUESTHANDLE;
	m_context.pObj			= this;

	if (!InternetReadFileEx(m_hRequest,
			  &InetBuff,
			  0,
			  (DWORD)&m_context))
	{
		if (GetLastError() == ERROR_IO_PENDING)
		{
			if(WaitForSingleObject(m_hRequestCompleteEvent, dwTimeOut) == WAIT_TIMEOUT)
				return FALSE;
		}
		else
			return FALSE;
	}


	return InetBuff.dwBufferLength;
}